////////////////////////////////////////////////////////////////////////////////
//
//  CLOUD CREATIVE STUDIOS, INC.
//
//  Copyright (C) 2011-2012 Cloud Creative Studios, Inc.
//  All Rights Reserved.
//
//  http://cloud-creative-studios.com
//
////////////////////////////////////////////////////////////////////////////////

ccs.HTTPClient = {};

(function() {
	
	var HTTPClient_Default_Timeout = 10 * 1000;
	var HTTPClient_Default_Retry = 3;
	
	//--------------------------------------------------------------------------
	ccs.HTTPClient.abort = function() {
		Ti.App.fireEvent("app:WendyClient_Abort");
	};
	
	//--------------------------------------------------------------------------
	///@brief リトライ付き非同期HTTPリクエスト
	///@param [in] params.options.method HTTPメソッド
	///@param [in] params.options.url リクエストURL
	///@param [in] params.options.enableKeepAlive Keep Alive ※省略可
	///@param [in] params.options.timeout タイムアウト(ms) ※省略可
	///@param [in] params.options.retry リトライ回数 ※省略可
	///@param [in] params.options.body 送信内容(連想配列)
	///@param [in] params.success リクエスト成功時のコールバック関数
	///@param [in] params.error リクエストエラー発生時のコールバック関数
	///
	///非同期HTTPリクエストをリトライ付きで実行する。
	///リクエストの完了は @p params.success をハンドリングすることで検出が可能。
	///
	///@attention
	///@p params.success と @p params.error に指定するコールバックの扱いに注意。
	///@p params.success は送信成功時やリトライを使い果たし場合など、
	///リクエスト発行が完了した時点で呼び出される。
	///
	///@p params.success に渡されるパラメータは以下の通り。
	///function successCallback(succeeded, xhr(省略あり));
	///@p succeeded にはHTTPの送信が成功した場合にtrueが指定される。
	///一方、HTTPリクエストが失敗した場合はfalseが指定される。
	///@p succeeded がtrueの場合は、xhrにステータスなどを保持する、
	///HTTPクライアントオブジェクトが渡される。
	///
	///@params.error は一回のHTTPリクエストが失敗するたびに呼び出される。
	///リトライ回数が10回の指定で呼び出した場合、
	///全てのリクエストが失敗した場合、@p params.error は10回呼び出される。
	ccs.HTTPClient.call = function(params) {
		var xhr;
		if (params.options.enableKeepAlive != undefined) {
			xhr = Ti.Network.createHTTPClient({
				enableKeepAlive:params.options.enableKeepAlive
			});
		} else {
			xhr = Ti.Network.createHTTPClient();
		}
		
		// タイムアウト及びリトライ回数は省略された場合はデフォルトの値を設定
		if (params.options.timeout == undefined) params.options.timeout = HTTPClient_Default_Timeout;
		if (params.options.retry == undefined) params.options.retry = HTTPClient_Default_Retry;
		
		var send = function(i) {
			ccs.DebugLog("ccs.HTTPClient.call -> sending[" + i + "]");
			ccs.DebugLog("ccs.HTTPClient.call -> " + params.options.method + " " + params.options.url);
			
			var deferred = new Deferred();
			xhr.open(params.options.method, params.options.url);
			xhr.setTimeout(params.options.timeout);
			if (params.options.headers != undefined) {
				params.options.headers.forEach(function(e, i, a) {
					xhr.setRequestHeader(e.key, e.value);
				});
			}
			xhr.onerror = function(e) {
				ccs.ErrorLog(e);
				if (params.error != undefined) params.error(e);
				deferred.call();
			};
			xhr.onreadystatechange = function(e) {
				ccs.DebugLog("ccs.HTTPClient.call -> Change state: " + this.readyState);
			};
			xhr.onload = function(e) {
				ccs.DebugLog("ccs.HTTPClient.call -> Response status: " + this.status);
				if (params.success != undefined) params.success(true, this);
			};
			if (params.options.body != undefined) {
				xhr.send(params.options.body);
			} else {
				xhr.send();
			}
			return deferred;
		};
		
		loop(params.options.retry, function(i) {
			return send(i);
		}).next(function() {
			if (params.success != undefined) params.success(false);
		});
	};
	//jsonに変換する
	ccs.HTTPClient.requestBasicJsonResponse = function(url, callback) {
		ccs.HTTPClient.requestBasicTextResponse({
			url:url,
			success:function(succeeded, responseText) {
				if (!!succeeded) {
					callback(true, JSON.parse(responseText));
				} else {
					callback(false, {});
				}
			},
			error:function(e) {
				callback(false, e);
			}
		});
	};
	//responseTextのまま
	ccs.HTTPClient.requestBasicTextResponse = function(params) {
		ccs.HTTPClient.call({
			options:{
				method:"GET",
				url:params.url,
			},
			success:function(succeeded, xhr) {
				if (!!succeeded) {
					params.success(true, xhr.responseText);
				} else {
					params.success(false);
				}
			},
			error:function(e) {
				params.error(e);
			}
		});
	};
}());
