////////////////////////////////////////////////////////////////////////////////
//
//  CLOUD CREATIVE STUDIOS, INC.
//
//  Copyright (C) 2011-2012 Cloud Creative Studios, Inc.
//  All Rights Reserved.
//
//  http://cloud-creative-studios.com
//
////////////////////////////////////////////////////////////////////////////////

var path_lib = '';
if (ccs.Aux.isIncludedOldStyle()){
  path_lib = '/lib/twitter/';
}
Ti.include(path_lib + "twitter_api.js");

///@brief Twitter API
///
///このAPIはTitaniumのFacebookと同様の利用方法で扱えるように、
///インターフェイスを同じような設計にしている。
ccs.Twitter = {};

(function() {
	
	//--------------------------------------------------------------------------
	///@brief Twitterに関するイベント登録
	///@param [in] eventName イベント名
	///@param [in] eventHandler イベントハンドラ
	ccs.Twitter.addEventListener = function(eventName, eventHandler) {
		Ti.App.addEventListener("TwitterEventHandler_" + eventName, eventHandler);
	};
	
	//--------------------------------------------------------------------------
	///@brief Twitterに関するイベント破棄
	///@param [in] eventName イベント名
	///@param [in] eventHandler イベントハンドラ
	ccs.Twitter.removeEventListener = function(eventName, eventHandler) {
		Ti.App.removeEventListener("TwitterEventHandler_" + eventName, eventHandler);
	};
	
	//--------------------------------------------------------------------------
	///@brief Twitter APIの初期化
	///@pre Ti.App.Twitterパラメータの設定
	///
	///Twitter APIを利用する場合は、必ず最初に呼び出しておく必要がある。
	///また、この関数の呼び出し前に予めTi.App.Twitterにパラメータを設定しておく必要がある。
	///
	/// - Ti.App.Twitter.ConsumerKey : コンシューマキー
	/// - Ti.App.Twitter.ConsumerSecret : コンシューマシークレット
	///
	///ログインを行うには @p ccs.Twitter.login() を呼び出す必要がある。
	///
	///@sa ccs.Twitter.login()
	ccs.Twitter.init = function() {
		// Twitterの認証
		Ti.App.twitterApi = new TwitterApi({
			consumerKey:Ti.App.Twitter.ConsumerKey,
			consumerSecret:Ti.App.Twitter.ConsumerSecret
		});
		twitterApi = Ti.App.twitterApi;
		twitterApi.init();
	};
	
	//--------------------------------------------------------------------------
	///@brief ログイン
	///@param [in] params.success ログイン成功時に呼び出されるコールバック関数
	///@param [in] params.error ログインエラー時に呼び出されるコールバック関数
	///@param [in] params.cancel ログインキャンセル時に呼び出されるコールバック関数
	///
	///この関数を呼び出すとログイン状態でない場合は、
	///ログインを行うためのウィンドウが開かれる。
	///既にログイン済みの場合は、何もせずに関数から戻る。
	///
	///ログアウトは @p ccs.Twitter.logout() を呼び出す。
	///
	///@sa ccs.Twitter.logout()
	ccs.Twitter.login = function(params) {
		var twitterApi = Ti.App.twitterApi;
		twitterApi.authorize(params);
	};
	
	//--------------------------------------------------------------------------
	///@brief Twitterボタンの作成
	///@param [in] params ボタン作成時のパラメータと同様
	ccs.Twitter.createLoginButton = function(params) {
		var setLogoutImage = function(obj) {
			obj.backgroundImage = "/lib/twitter/images/sign-in-with-twitter-l.png";
			obj.backgroundSelectedImage = "/lib/twitter/images/sign-in-with-twitter-l_press.png";
		};
		var setLoggedinImgae = function(obj) {
			obj.backgroundImage = "/lib/twitter/images/sign-in-with-twitter-l_loggedin.png";
			obj.backgroundSelectedImage = "/lib/twitter/images/sign-in-with-twitter-l_loggedin.png";
		};
		
		params.width = 151;
		params.height = 24;
		
		if (ccs.Twitter.isLoggedIn()) {
			setLoggedinImage(params);
		} else {
			setLogoutImage(params);
		}
		
		var button = Ti.UI.createButton(params);
		button.addEventListener("click", function(e) {
			if (ccs.Twitter.isLoggedIn()) {
				ccs.Twitter.logout();
				setLogoutImage(button);
			} else {
				var win = Ti.UI.createWindow({
					top:0, left:0, width:"100%", height:"100%"
				});
				win.open();
				var closeWindow = function() {
					win.hide();
					win.close();
					win = null;
				};
				var eventHandler = function(e) {
					ccs.Twitter.removeEventListener("login", eventHandler);
					closeWindow();
					if (e.success != undefined) {
						setLoggedinImgae(button);
					}
				};
				ccs.Twitter.addEventListener("login", eventHandler);
				ccs.Twitter.login();
			}
		});
		return button;
	};
	
	//--------------------------------------------------------------------------
	///@brief ログアウト
	///
	///この関数を呼び出すとログアウトを行う。
	///既にログアウトの場合は、何もせずに関数から戻る。
	///
	///@sa ccs.Twitter.login()
	ccs.Twitter.logout = function() {
		var twitterApi = Ti.App.twitterApi;
		twitterApi.clear_accesstoken();
		Ti.App.fireEvent("TwitterEventHandler_logout");
	};
	
	//--------------------------------------------------------------------------
	///@brief ログイン状態判定
	///@return ログイン状態
	///@retval true ログイン中
	///@retval false ログアウト又は初期化されていない
	ccs.Twitter.isLoggedIn = function() {
		if (Ti.App.twitterApi != undefined) {
			var twitterApi = Ti.App.twitterApi;
			if (twitterApi.isAuthorized()) {
				return true;
			}
		}
		
		return false;
	};
	
	//--------------------------------------------------------------------------
	///@brief スクリーン名を返す
	///@return スクリーン名
	ccs.Twitter.getScreenName = function() {
		var twitterApi = Ti.App.twitterApi;
		return twitterApi.oAuthAdapter.getScreenName();
	};
	
	//--------------------------------------------------------------------------
	///@brief ツイート検索
	///@param [in] params.q 検索ワード
	///@param [in] params.rpp 検索最大件数
	///@param [in] callbackHandler 検索完了後のコールバック関数
	ccs.Twitter.search = function(params, callbackHandler) {
		var twitterApi = Ti.App.twitterApi;
		var deferred = new Deferred();
		twitterApi.search({
			parameters:params,
			onSuccess:function(response) {
				callbackHandler({
					success:true,
					response:response
				});
				deferred.call(response);
			},
			onError:function(e) {
				callbackHandler({
					success:false,
					error:e.error
				});
				deferred.fail();
			}
		});
		return deferred;
	};
	
	//--------------------------------------------------------------------------
	///@brief プロフィール画像を返す
	///@param [in] params.name スクリーン名
	///@param [in] callbackHandler リクエスト完了時のコールバック関数
	///
	///コールバック関数は以下のようなパラメータで呼び出される。
	/// - success : 投稿成功(true:成功, false:失敗)
	/// - image : 画像Blob(成功時のみ)
	ccs.Twitter.getProfile = function(params, callbackHandler) {
		var twitterApi = Ti.App.twitterApi;
		twitterApi.users_lookup({
			parameters:{
				screen_name:params.name
			},
			onSuccess:function(response) {
				callbackHandler({
					success:true,
					response:response
				});
			},
			onError:function(e) {
				callbackHandler({
					success:false,
					error:e.error
				});
			}
		});
	};
	
	//--------------------------------------------------------------------------
	ccs.Twitter.getProfileImage = function(params, callbackHandler) {
		var twitterApi = Ti.App.twitterApi;
		twitterApi.users_profile_image_twitter({
			parameters:{
				screen_name:params.name
			},
			onSuccess:function(response, data) {
				callbackHandler({
					success:true,
					image:data
				});
			},
			onError:function(e) {
				callbackHandler({
					success:false
				});
			}
		});
	};
	
	//--------------------------------------------------------------------------
	///@brief Twitterへの投稿
	///@param [in] params Twitterのstatus/updateパラメータ
	///@param [in] callbackHandler 投稿完了時のコールバック関数
	///
	///コールバック関数は以下のようなパラメータで呼び出される。
	/// - success : 投稿成功(true:成功, false:失敗)
	/// - status : HTTPステータスメッセージ
	///
	///画像付きツイートは ccs.Twitter.tweetWithImage() を参照。
	///
	///@p params のパラメータに関しては以下を参照。
	///https://dev.twitter.com/docs/api/1/post/statuses/update 
	///
	///@sa ccs.Twitter.tweetWithImage()
	ccs.Twitter.tweet = function(params, callbackHandler) {
		var twitterApi = Ti.App.twitterApi;
		
		// Twitterへの投稿
		twitterApi.statuses_update({
			onSuccess: function(response) {
				callbackHandler({
					success:true
				});
			},
			onError: function(e) {
				var status;
				
				// iPhoneとAndroidでは返されるエラー内容が違う!!
				// ってか、こんな仕様は何とかして欲しい...
				if (ccs.Aux.Platform.isAndroid()) {
					status = e.error;
				} else {
					// 返されるのは
					// source = "[object TiNetworkClient]"
					// type = error;
					// だけ
					// var code = e.error.match(/[0-9]+[0-9]+[0-9]+/i);
					// if (code == "403") {
						// status = "Forbidden";
					// } else {
						// status = "Unknown";
					// }
					status = "Unknown";
				}
				callbackHandler({
					success:false,
					status:status
				});
			},
			parameters:params
		});
	};
	
	//--------------------------------------------------------------------------
	///@brief Twitterへの投稿(画像付き)
	///@param [in] params.picture 投稿する画像(Blob)
	///@param [in] callbackHandler 投稿完了時のコールバック関数
	///
	///基本的にな機能は ccs.Twitter.tweet() と同じだが、
	///この関数はツイート前に TwitPic に対して画像のアップロードを行い、
	///Twitterへのツイート時のメッセージの末尾に、
	///アップロードされた画像のURL(短縮URL)が付与される。
	///
	///@p params のpicture以外のパラメータは ccs.Twitter.tweet() を参照。
	///
	///@sa ccs.Twitter.tweet(), ccs.Twitter.uploadToTwitPic()
	ccs.Twitter.tweetWithImage = function(params, callbackHandler) {
		// 現在のコンテキスト(this)を保存しておき、
		// ccs.Twitter.uploadToTwitPic()内で呼び出せるようにする
		var thiscopy = this;
		
		this.uploadToTwitPic(params, function(e) {
			if (e.success) {
				var temp = {
					message:(params.message || "") + " " + e.response.url
				};
				thiscopy.tweet(temp, callbackHandler);
			} else {
				callbackHandler(e);
			}
		});
	};
	
	//--------------------------------------------------------------------------
	///@brief TwitPicへの画像投稿
	///@param [in] params.message 画像と合わせて投稿するメッセージ
	///@param [in] params.picture 投稿する画像(Blob)
	///@param [in] callbackHandler 投稿が完了した時点で呼び出されるコールバック関数
	///@pre Ti.App.Twitter.twitPicKeyの設定
	///
	///TwitPicへの画像投稿の機能を提供する。
	///
	///この関数を呼び出すには事前に Ti.App.Twitter.twitPicKey に対して、
	///有効なキーを設定しておく必要がある。
	///
	///@note
	///Twitpicへの投稿は現在の仕様では事前にTwitter連携を行なっておく必要がある。
	///認証URL：http://twitpic.com/session/new
	///
	///@sa ccs.Twitter.tweetWithImage()
	ccs.Twitter.uploadToTwitPic = function(params, callbackHandler) {
		var twitterApi = Ti.App.twitterApi;
		var verifyURL = "https://api.twitter.com/1/account/verify_credentials.json";
		var option = {
			url:verifyURL,
			method:"GET"
		};
		var header = twitterApi.oAuthAdapter.createOAuthHeader(option);
		var headers = [
			{ key:"X-Verify-Credentials-Authorization", value:header },
			{ key:"X-Auth-Service-Provider", value:verifyURL }
		];
		ccs.HTTPClient.call({
			options:{
				headers:headers,
				enableKeepAlive:false,
				url:"http://api.twitpic.com/2/upload.json",
				method:"POST",
				body:{
					key:Ti.App.Twitter.TwitPicKey,
					message:params.message,
					media:params.picture
				}
			},
			success:function(succeeded, xhr) {
				if (!!succeeded && xhr.status == 200) {
					var res = JSON.parse(xhr.responseText);
					callbackHandler({
						success:true,
						status:xhr.status,
						response:res
					});
				} else {
					callbackHandler({
						success:false,
						status:xhr.status
					});
				}
			}
		});
	};
	//--------------------------------------------------------------------------
	///@brief リストを作成する
	///@param [in] params.name リスト名
	///@param [in] params.mode public,private
	///@param [in] params.description リストの説明
	///@param [in] success 成功時のコールバック
	///@param [in] error エラー時のコールバック
	///@p リクエスト制限なし
	ccs.Twitter.createList = function(params) {
		var twitterApi = Ti.App.twitterApi;
		twitterApi.user_lists_create({
			parameters:{
				name:params.name,
				mode:params.mode,
				description:params.description
			},
			onSuccess:params.success,
			onError:params.error
		});
	};
	//--------------------------------------------------------------------------
	///@brief リストを取得する
	///@param [in] params.slug リスト名
	///@param [in] params.ownerScreenName ユーザ名
	///@param [in] params.ownerId ユーザID
	///@p リクエスト制限あり
	ccs.Twitter.showList = function(params) {
		var twitterApi = Ti.App.twitterApi;
		twitterApi.user_lists_get({
			parameters:{
				slug:params.slug,
				owner_screen_name:ccs.Twitter.getScreenName()
			},
			onSuccess:params.success,
			onError:params.error
		});
	};
	//--------------------------------------------------------------------------
	///@brief 指定したユーザがリスト内のメンバーかどうかを調べる
	///@param [in] params.slug リスト名
	///@param [in] params.screenName 調べる対象のユーザ名 
	///@p リクエスト制限あり
	ccs.Twitter.showListMember = function(params) {
		var twitterApi = Ti.App.twitterApi;
		twitterApi.user_lists_members_show({
			parameters:{
				slug:params.slug,
				owner_screen_name:ccs.Twitter.getScreenName(),
				screen_name:params.screenName
			},
			onSuccess:params.success,
			onError:params.error
		});
	};
	//--------------------------------------------------------------------------
	///@brief リスト内メンバー一覧を取得する
	///@param [in] params.slug リスト名
	///@param [in] params.cursor 取得ページ(1ページあたり20件)
	///@p リクエスト制限あり
	ccs.Twitter.getListMembers = function(params) {
		var twitterApi = Ti.App.twitterApi;
		twitterApi.user_lists_of_user({
			parameters:{
				slug:params.slug,
				owner_screen_name:ccs.Twitter.getScreenName(),
				cursor:params.cursor
			},
			onSuccess:params.success,
			onError:params.error
		});
	};
	//--------------------------------------------------------------------------
	///@brief リスト内メンバーのつぶやきタイムラインを取得する
	///@param [in] params.slug リスト名
	///@param [in] params.page 取得ページ
	///@p リクエスト制限あり
	ccs.Twitter.getListStatuses = function(params) {
		var twitterApi = Ti.App.twitterApi;
		twitterApi.user_lists_statuses({
			parameters:{
				slug:params.slug,
				owner_screen_name:ccs.Twitter.getScreenName(),
				page:params.page
			},
			onSuccess:params.success,
			onError:params.error
		});
	};
	//--------------------------------------------------------------------------
	///@brief 指定したリストにメンバーを作成（追加）する
	///@param [in] params.slug リスト名
	///@param [in] userId 作成するメンバーのユーザID
	///@param [in] screenName 作成するメンバーのユーザ名
	///@p リクエスト制限なし
	ccs.Twitter.createListMember = function(params) {
		var twitterApi = Ti.App.twitterApi;
		twitterApi.user_lists_members_create({
			parameters:{
				slug:params.slug,
				owner_screen_name:ccs.Twitter.getScreenName(),
				user_id:params.userId,
				screen_name:params.screenName
			},
			onSuccess:params.success,
			onError:params.error
		});
	};
	//--------------------------------------------------------------------------
	///@brief 指定したリストからメンバーを削除する
	///@param [in] params.slug リスト名
	///@param [in] screen_name 削除するメンバーのユーザ名
	///@p リクエスト制限なし
	ccs.Twitter.deleteListMember = function(params) {
		var twitterApi = Ti.App.twitterApi;
		twitterApi.user_lists_members_delete({
			parameters:{
				slug:params.slug,
				owner_screen_name:ccs.Twitter.getScreenName(),
				user_id:params.userId,
				screen_name:params.screenName
			},
			onSuccess:params.success,
			onError:params.error
		});
	};
	
}());
